#!/usr/bin/env python

import cherrypy
import json
from dm.common.utility.encoder import Encoder
from dm.common.service.dmController import DmController
from dm.cat_web_service.service.impl.fileControllerImpl import FileControllerImpl

class FileController(DmController):

    def __init__(self):
        DmController.__init__(self)
        self.fileControllerImpl = FileControllerImpl()

    @cherrypy.expose
    @DmController.execute
    def addExperimentFile(self, experimentName, fileName, **kwargs):
        if not experimentName:
            raise InvalidRequest('Invalid experiment name provided.')
        if not fileName:
            raise InvalidRequest('Invalid file name provided.')
        encodedFileInfo = kwargs.get('fileInfo')
        if not encodedFileInfo:
            raise InvalidRequest('Invalid file info provided.')
        fileInfo = json.loads(Encoder.decode(encodedFileInfo))
        fileInfo['name'] = fileName
        fileInfo['experimentName'] = experimentName
        response = self.fileControllerImpl.addExperimentFile(fileInfo).getFullJsonRep()
        self.logger.debug('Added file %s: %s' % (fileName,response))
        return response

    @cherrypy.expose
    @DmController.execute
    def updateExperimentFile(self, experimentName, fileName, **kwargs):
        if not experimentName:
            raise InvalidRequest('Invalid experiment name provided.')
        if not fileName:
            raise InvalidRequest('Invalid file name provided.')
        encodedFileInfo = kwargs.get('fileInfo')
        if not encodedFileInfo:
            raise InvalidRequest('Invalid file info provided.')
        fileInfo = json.loads(Encoder.decode(encodedFileInfo))
        fileInfo['name'] = fileName
        fileInfo['experimentName'] = experimentName
        response = self.fileControllerImpl.updateExperimentFile(fileInfo).getFullJsonRep()
        self.logger.debug('Updated file %s: %s' % (fileName,response))
        return response

    @cherrypy.expose
    @DmController.execute
    def updateFileById(self, id, **kwargs):
        if not id:
            raise InvalidRequest('Invalid file id provided.')
        encodedFileInfo = kwargs.get('fileInfo')
        if not encodedFileInfo:
            raise InvalidRequest('Invalid file info provided.')
        fileInfo = json.loads(Encoder.decode(encodedFileInfo))
        response = self.fileControllerImpl.updateFileById(fileInfo).getFullJsonRep()
        self.logger.debug('Updated file id %s: %s' % (id,response))
        return response

    @cherrypy.expose
    @DmController.execute
    def getFiles(self, **kwargs):
        encodedQueryDict = kwargs.get('queryDict')
        queryDict = {}
        if encodedQueryDict:
            queryDict = json.loads(Encoder.decode(encodedQueryDict))
        return self.listToJson(self.fileControllerImpl.getFiles(queryDict=queryDict))

    @cherrypy.expose
    @DmController.execute
    def getExperimentFiles(self, experimentName, **kwargs):
        if not experimentName:
            raise InvalidRequest('Invalid experiment name provided.')
        encodedQueryDict = kwargs.get('queryDict')
        queryDict = {}
        if encodedQueryDict:
            queryDict = json.loads(Encoder.decode(encodedQueryDict))
        return self.listToJson(self.fileControllerImpl.getExperimentFiles(experimentName, queryDict=queryDict))

    @cherrypy.expose
    @DmController.execute
    def getFileById(self, id, **kwargs):
        if not id:
            raise InvalidRequest('Invalid id provided.')
        response = self.fileControllerImpl.getFileById(id).getFullJsonRep()
        self.logger.debug('Returning file id %s: %s' % (id,response))
        return response

    @cherrypy.expose
    @DmController.execute
    def getExperimentFile(self, experimentName, fileName, **kwargs):
        if not experimentName:
            raise InvalidRequest('Invalid experiment name provided.')
        if not fileName:
            raise InvalidRequest('Invalid file name provided.')
        response = self.fileControllerImpl.getExperimentFile(experimentName, fileName).getFullJsonRep()
        self.logger.debug('Returning file %s: %s' % (fileName,response))
        return response

