#!/usr/bin/env python

import copy
from dm.common.exceptions.dmException import DmException
from dm.common.mongodb.api.dmMongoDbApi import DmMongoDbApi
from dm.common.mongodb.impl.fileCollection import FileCollection
from dm.common.mongodb.api.dmMongoDbApi import DmMongoDbApi
from dm.common.objects.fileMetadata import FileMetadata

class FileMongoDbApi(DmMongoDbApi):

    def __init__(self):
        DmMongoDbApi.__init__(self)
        self.fileCollection = FileCollection(self.dbClient)

    @DmMongoDbApi.executeDbCall
    def addExperimentFile(self, fileInfo, **kwargs):
        dbFileMetadata = self.fileCollection.addByUniqueKeys(fileInfo)
        return self.toDmObject(dbFileMetadata, FileMetadata)

    @DmMongoDbApi.executeDbCall
    def getFiles(self, queryDict={}, returnFieldDict=FileCollection.ALL_FIELDS_DICT, **kwargs):
        return self.listToDmObjects(self.fileCollection.findByQueryDict(queryDict, returnFieldDict), FileMetadata)

    @DmMongoDbApi.executeDbCall
    def getFileById(self, id, **kwargs):
        dbFileMetadata = self.fileCollection.findById(id)
        return self.toDmObject(dbFileMetadata, FileMetadata)

    @DmMongoDbApi.executeDbCall
    def getExperimentFile(self, experimentName, fileName, **kwargs):
        queryDict = { 'name' : fileName, 'experimentName' : experimentName }
        dbFileMetadata = self.fileCollection.findByUniqueKeys(queryDict)
        return self.toDmObject(dbFileMetadata, FileMetadata)

    @DmMongoDbApi.executeDbCall
    def getExperimentFiles(self, experimentName, queryDict={}, returnFieldDict=FileCollection.ALL_FIELDS_DICT, **kwargs):
        queryDict2 = copy.copy(queryDict)
        queryDict2['experimentName'] = experimentName 
        return self.getFiles(queryDict2, returnFieldDict)

    @DmMongoDbApi.executeDbCall
    def updateFileById(self, fileInfo, **kwargs):
        dbFileMetadata = self.fileCollection.updateById(fileInfo)
        return self.toDmObject(dbFileMetadata, FileMetadata)

    @DmMongoDbApi.executeDbCall
    def updateExperimentFile(self, fileInfo, **kwargs):
        dbFileMetadata = self.fileCollection.updateByUniqueKeys(fileInfo)
        return self.toDmObject(dbFileMetadata, FileMetadata)

    @DmMongoDbApi.executeDbCall
    def updateOrAddExperimentFile(self, fileInfo, **kwargs):
        dbFileMetadata = self.fileCollection.updateOrAddByUniqueKeys(fileInfo)
        return self.toDmObject(dbFileMetadata, FileMetadata)

#######################################################################
# Testing.
if __name__ == '__main__':
    api = FileMongoDbApi()
    files = api.getFiles()
    for file in files:
        print file.getDictRep()
        print file.__dict__
    file = api.getExperimentFile('exp-01', 'xyz-001')
    print file
    import time
    t = long(time.time())
    print t
    fileName = 'f-%s' % t
    fileInfo = {'name' : fileName, 'intKey' : 1, 'doubleKey' : 2.0, 'stringKey' : 'myString' , 'dictKey' : {'a' : 'A', 'b' : 'B', 'c' : 3}, 'experimentName' : 'exp2'}
    file = api.addExperimentFile(fileInfo)
    print '\nADDED FILE\n', file


    fileInfo = {'name' : fileName, 'intKey' : 101}
    file = api.updateExperimentFile(fileInfo)
    print '\nUPDATED FILE\n', file
    print '\nFILES: \n', api.getFiles()
    print '\nFILES FOR EXPERIMENT exp1: \n', api.getFiles(queryDict={'experimentName' : 'exp1'})


